TeaVM
=====

[![Build Status](https://travis-ci.org/konsoletyper/teavm.svg?branch=master)](https://travis-ci.org/konsoletyper/teavm) 
[![Maven Central](https://maven-badges.herokuapp.com/maven-central/org.teavm/teavm-maven-plugin/badge.svg)](https://maven-badges.herokuapp.com/maven-central/org.teavm/teavm-maven-plugin) 
[![Bintray](https://api.bintray.com/packages/konsoletyper/teavm/teavm-dev/images/download.svg)](https://bintray.com/konsoletyper/teavm/teavm-dev/_latestVersion)
[![Gitter chat](https://img.shields.io/badge/gitter-join%20chat-green.svg)](https://gitter.im/teavm/Lobby)

What is TeaVM?
--------------

TeaVM is an ahead-of-time compiler of Java bytecode.
Currently, is supports translation to JavaScript and WebAssembly (experimental).
It can be compared with GWT, however TeaVM does not require source code of your application and
all required libraries.
You can use TeaVM for building applications for the browser, due to the following features:

  * per-method dependency analyzer, that determines a set of methods that are really needed
    to run your application, so TeaVM won't translate whole JAR files;
  * fast JavaScript; for now it is almost as fast as the JavaScript, generated by GWT;
  * Java class library emulation;
  * integration with Maven and Eclipse;
  * generation of source maps;
  * debugger;
  * interoperation with JavaScript libraries together with the set of predefined browser interfaces.
  * supports WebAssembly output (experimental).


Quick start
-----------

There are several options of using TeaVM. One is the Maven build.
The easiest way to create a new TeaVM project is to type in the command line:

    mvn -DarchetypeCatalog=local \
      -DarchetypeGroupId=org.teavm \
      -DarchetypeArtifactId=teavm-maven-webapp \
      -DarchetypeVersion=0.4.3 archetype:generate

Now you can execute `mvn clean package` and get the generated `war` file.
Deploy this `war` in Tomcat or another container, or simply unzip it and open the `index.html` page.

To learn TeaVM deeper, you take a look at the [samples](samples) module,
containing examples of TeaVM-based projects.
Also you can read [project's wiki](https://github.com/konsoletyper/teavm/wiki/).


Preview builds
--------------

You may want to access new features and don't want to wait until stable release is published on Maven Central. In this case you can get latest development build from [bintray](https://bintray.com/konsoletyper/teavm/teavm-dev). All you need is to put the following in your `pom.xml`:

```xml
  <repositories>
    <repository>
      <id>teavm-dev</id>
      <url>https://dl.bintray.com/konsoletyper/teavm</url>
    </repository>
  </repositories>
```

IDEA plugin is also available in preview builds. You need to add corresponding repository manually to IDEA. Open *Settings* -> *Plugins* -> *Browse repositories...* -> *Manage repositories...*, click *Add* button and enter `http://teavm.org/idea/dev/teavmRepository.xml`. Then get back to *Browse repositories* and pick TeaVM plugin from list. 


Building TeaVM
--------------

Simply clone source code (`git clone https://github.com/konsoletyper/teavm.git`) and run maven build (`mvn clean install`). You can do things a little faster (`mvn clean -DskipTests`) or even a little more faster (`mvn clean -DskipTests -Dteavm.build.all=false`).


Embedding TeaVM
---------------

If you are not satisfied with Maven, you can embed TeaVM in your program or even create your own plugin for any build tool, like Ant or Gradle. The starting point for you may be `org.teavm.tooling.TeaVMTool` class from `teavm-tooling` artifact. You may want to go deeper and use `org.teavm.vm.TeaVM` from `teavm-core` artifact, learn how `TeaVMTool` initializes it. To learn how to use `TeaVMTool` class itself, find its usages across project source code. You most likely encounter Maven and IDEA plugins.
  
Please, notice that these APIs for embedding are still unstable and may change between versions.


WebAssembly
-----------

WebAssembly support is in experimental status. It may lack major features available in JavaScript backend. There's no documentation yet and you should do many things by hands (like embedding generated `wasm` file into your page, importing JavaScript objects, etc). Look at [samples/benchmark](https://github.com/konsoletyper/teavm/blob/master/samples/benchmark/) module. You should first examine `pom.xml` file to learn how to build `wasm` file from Java. Then you may want to examine `index-teavm.html` and `index-teavm.js` to learn how to embed WebAssembly into your web page. 


Feedback
--------

More information is available at the official site: http://teavm.org.

Ask your questions by email: info@teavm.org. Also you can report issues on a project's [issue tracker](https://github.com/konsoletyper/teavm/issues).
